//******************************************************************
//* EmbedRF API
//* LAST UPDATED 08/15/2008
//* FIRMWARE BY FRED EADY FOR SERVO MAGAZINE
//* CHANGES/ADDITIONS
//******************************************************************
#include <pic18.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include <htc.h>
#include <stdlib.h>

// Settings for PIC18F2620
__CONFIG(1,IESODIS & FCMDIS & RCIO);
__CONFIG(2,BORDIS & PWRTDIS & WDTDIS);
__CONFIG(3,MCLREN & LPT1DIS & PBDIGITAL);
__CONFIG(4,DEBUGEN & LVPDIS & XINSTDIS & STVREN);
__CONFIG(5,UNPROTECT);
__CONFIG(6,WRTEN);
__IDLOC(0815);

//******************************************************************
//*	FUNCTION PROTOTYPES
//******************************************************************
void get_firmware_version(void);
void get_tx_interval(void);
char set_tx_interval(char interval);
char set_tx_rx_mode(char mode);
void get_tx_mode(void);
void get_embedrf_data(void);
char send_data_now(char cmd);
char send_data_buffer(char mode);
char set_tx_pwr_lvl(char level);
void get_tx_pwr_lvl(void);
char set_device_id(char id_ww, char id_xx, char id_yy, char id_zz);
void get_device_id(char device);
char set_ad_settings(char ad0val, char ad1val, char ad2val, char ad3val);
void get_ad_settings(void);
void find_xmitr(char mode);
char set_network_id(char nid_xx, char nid_yy, char nid_zz);
char set_notify_mode(char mode);
void set_id_random(char mode);
char set_network_mode(char mode);
void get_network_mode(void);
void interrupt EUSART_TIMER(void);
void init(void);
void putch(char c);
unsigned int recvchar(void);
unsigned int sendchar(unsigned int data);
char CharInQueue(void);
void initEUSART(void);
void cleaner(void);
//******************************************************************
//*	PIC18F2620 RAM Definitions
//******************************************************************
char usart_data,tmphead,tmptail,temp_8;
char rc;
char hours3, mins3, secs3;
unsigned int msecs3,tmsecs3,tsecs3;

#define	err_led	LATA5
#define	ON	0x00
#define OFF 0x01

typedef struct {
					char	ver_maj:8;		
					char	ver_min:8;
					char	tx_interval:8;
					char	tx_mode:8;
					char	tx_pwr_lvl:8;
					char	device_id_aa:8;
					char	device_id_bb:8;
					char	device_id_cc:8;
					char	ad_setting_ww:8;
					char	ad_setting_xx:8;
					char	ad_setting_yy:8;
					char	ad_setting_zz:8;
					char	find_tx_xx:8;
					char	find_tx_yy:8;
					char	find_tx_zz:8;
					char	rid_xx:8;
					char	rid_yy:8;
					char	rid_zz:8;
					char	network_mode:8; 
				} RReply;

RReply reply;

//******************************************************************
//    MACROS 
//******************************************************************
#define bitset(var, bitno) ((var) |= 1 << (bitno))
#define bitclr(var, bitno) ((var) &= ~(1 << (bitno)))
#define make8(var,offset)	((unsigned int)var >> (offset * 8)) & 0x00FF
#define	make16(varhigh,varlow)	(((unsigned int)varhigh & 0xFF)* 0x100) + ((unsigned int)varlow & 0x00FF)
#define make32(var1,var2,var3,var4) \
		((unsigned long)var1<<24)+((unsigned long)var2<<16)+ \
		((unsigned long)var3<<8)+((unsigned long)var4)		
#define make32i(var1,var2) ((unsigned long)var1<<16)+((unsigned long)var2)
//******************************************************************
//*	EmbedRF COMMAND/REPLY DEFINITONS
//******************************************************************

//Transmit/Receive Mode Configuration Options
#define	tr_off			0x00	//off - no transmit or receive
#define tr_rcv			0x01	//receive only
#define tr_xmit			0x02	//transmit only
#define tr_rx			0x03	//receive then transmit
#define tr_xr			0x04	//transmit then receive

//Send Data Immediately Options
#define xmit_only		0x00
#define	xmit_rcv		0x01

//Send Data to Buffer Options
#define	no_xmit			0x00
#define xmit_now		0x01

//Transmit Power Level Configuration Options
#define	dbm_minus10		0x00	//-10 dbm
#define dbm_minus5		0x01	//-5 dbm
#define	dbm_0			0x02	//0 dbm
#define	dbm_plus5		0x03	//5 dbm

//Device ID Specifier Definition
#define id_rcv			0x00
#define id_xmit			0x01

//Get Device ID Device Types
#define	rx_device		0x00
#define tx_device		0x01

//Analog/Digital Options
#define	ad_disabled		0x00	//disabled
#define	ad_enabled		0x01	//analog active
#define	dig_1			0x02	//digital active logic 1
#define dig_0			0x03	//digital active logic 0

//Find Transmitter Options
#define	find_first		0x00	//find first
#define prox_find		0x01	//proximity find


//Notify Mode Options
#define notify_off		0x00
#define	notify_on		0x01

//Pseudo-Random Options
#define	rcv_random		0x00
#define	xmit_random		0x01

//Network Mode Options
#define	p_to_p			0x00	//point to point
#define p_to_m			0x01	//point to multipoint

//******************************************************************
//*	DATA PACKET DEFINITONS
//******************************************************************
char packet_tx[11];
#define	ad0_hi				0x00
#define tx_0				0x00
#define ad0_lo				0x01
#define tx_1				0x01
#define	ad1_hi				0x02
#define tx_2				0x02
#define ad1_lo				0x03
#define tx_3				0x03
#define ad2_hi				0x04
#define tx_4				0x04
#define ad2_lo				0x05
#define tx_5				0x05
#define ad3_hi				0x06
#define tx_6				0x06
#define ad3_lo				0x07
#define tx_7				0x07
#define	tx_8				0x08
#define tx_9				0x09
#define tx_cmd				0x0A
#define tx_end				0x0B

char packet_rx[15];
#define xid_hi				0x00
#define xid_mid				0x01
#define xid_lo				0x02
#define rx_0				0x03
#define rx_1				0x04
#define rx_2				0x05
#define rx_3				0x06
#define rx_4				0x07
#define rx_5				0x08
#define rx_6				0x09
#define rx_7				0x0A
#define rx_8				0x0B
#define rx_9				0x0C
#define rx_rssi				0x0D
#define rx_ctr				0x0E
#define rx_end				0x0F
//******************************************************************
//*	INTERRUPT DEFINITIONS
//******************************************************************
#define		disable_RXint	  	RCIE = 0
#define		enable_RXint	  	RCIE = 1
#define		disable_TMR2int  	TMR2IE = 0
#define		enable_TMR2int	 	TMR2IE = 1
#define		disable_TMR3int  	TMR3IE = 0
#define		enable_TMR3int	 	TMR3IE = 1
#define		disable_TMR1int  	TMR1IE = 0
#define		enable_TMR1int	 	TMR1IE = 1
#define		disable_TMR0int  	TMR0IE = 0
#define		enable_TMR0int	 	TMR0IE = 1

#define		enable_GLOBALint  	IPEN = 0;				\
							  	GIE = 1;				\
							  	PEIE = 1;
#define		disable_GLOBALint 	IPEN = 0;				\
							  	GIE = 0;				\
							  	PEIE = 0;
#define		TIMER0ON			TMR0ON = 1
#define		TIMER0OFF			TMR0ON = 0
#define		TIMER1ON			TMR1ON = 1
#define		TIMER1OFF			TMR1ON = 0
#define		TIMER2ON		  	TMR2ON = 1
#define		TIMER2OFF		  	TMR2ON = 0
#define		TIMER3ON		  	TMR3ON = 1
#define		TIMER3OFF		  	TMR3ON = 0

#define delay_ms(msecdelay)		TIMER3OFF;					\
								TMR3IF = 0;					\
								msecs3 = 0;					\
								tmsecs3 = 0;				\
								TIMER3ON;					\
								while(tmsecs3 < msecdelay);
#define	delay_sec(secdelay)		TIMER3OFF;					\
								TMR3IF = 0;					\
								msecs3 = 0;					\
								tmsecs3 = 0;				\
								secs3 = 0;					\
								tsecs3 = 0;					\
								TIMER3ON;					\
								while(tsecs3 < secdelay);

//******************************************************************
//*	EUSART BUFFER DEFINITIONS
//******************************************************************
//1,2,4,8,16,32,64,128 or 256 bytes 
#define EUSART_RX_BUFFER_SIZE  32     
#define EUSART_RX_BUFFER_MASK ( EUSART_RX_BUFFER_SIZE - 1 )
// 1,2,4,8,16,32,64,128 or 256 bytes 
#define EUSART_TX_BUFFER_SIZE  32      
#define EUSART_TX_BUFFER_MASK ( EUSART_TX_BUFFER_SIZE - 1 )
unsigned char EUSART_RxBuf[EUSART_RX_BUFFER_SIZE],EUSART_TxBuf[EUSART_TX_BUFFER_SIZE];
unsigned char EUSART_TxHead,EUSART_TxTail,EUSART_RxHead,EUSART_RxTail;

void cleaner(void) //loads receive buffer with 0x00
{
	unsigned int x;
	for(x=0;x<EUSART_RX_BUFFER_SIZE-1;++x)
		EUSART_RxBuf[x] = 0;
	for(x=0;x<EUSART_TX_BUFFER_SIZE-1;++x)
		EUSART_TxBuf[x] = 0;
  	EUSART_RxTail = 0x00;	//flush receive buffer 
  	EUSART_RxHead = 0x00;
  	EUSART_TxTail = 0x00;	//flush transmit buffer
  	EUSART_TxHead = 0x00;
}
	
void putch(char c)
{
   while(!TRMT) 	// TRMT is set when TSR is empty
      continue;

  TXREG = c;		// load the register
}
unsigned int recvchar(void)
{
  unsigned char tmptail;
                                // wait for incomming usart_data  
  while ( EUSART_RxHead == EUSART_RxTail );
                                // calculate buffer index  
  tmptail = ( EUSART_RxTail + 1 ) & EUSART_RX_BUFFER_MASK;
  EUSART_RxTail = tmptail;        // store new index  

  return EUSART_RxBuf[tmptail];   // return usart_data  
}

unsigned int sendchar(unsigned int usart_data)
{
  unsigned char tmphead;
                                // calculate buffer index 
  tmphead = ( EUSART_TxHead + 1 ) & EUSART_TX_BUFFER_MASK;
                                // wait for free space in buffer 
  while ( tmphead == EUSART_TxTail );
                                // store usart_data in buffer 
  EUSART_TxBuf[tmphead] = (unsigned char)usart_data;
  EUSART_TxHead = tmphead;        // store new index 

  TXIE = 1;            // enable TX interrupt 

  return usart_data;
}

unsigned char CharInQueue(void)
{
  return(EUSART_RxHead != EUSART_RxTail);
}
//******************************************************************
//*	Init EUSART Function
//******************************************************************
void init_EUSART(void)
{
  SPBRG = 25;	//8MHZ = 25 - 4MHZ = 12 FOR 19200BAUD
  TRISC7 = 1;	//receive pin
  TRISC6 = 0;	//transmit pin
  TXSTA = 0x04;	//high speed baud rate	BRGH = 1
  RCSTA = 0x80;	//enable serial port and serial port pins
  
  EUSART_RxTail = 0x00;	//flush receive buffer 
  EUSART_RxHead = 0x00;
  EUSART_TxTail = 0x00;	//flush transmit buffer
  EUSART_TxHead = 0x00;
  RCIP = 1;			//receive interrupt = high priority
  TXIP = 1;			//transmit interrupt = high priority
  RCIE = 1;			//enable receive interrupt
  PEIE = 1;			//enable all unmasked peripheral interrupts
  GIE = 1;			//enable all unmasked interrupts
  CREN = 1;			//enable EUSART1 receiver
  TXIE = 0;			//disable EUSART1 transmit interrupt
  TXEN = 1;			//transmitter enabled
}
void interrupt EUSART_TIMER(void)
{
//SERVICE THE TIMER
	if((TMR3IF && TMR3IE))
	{
		TMR3IF = 0;
		TMR3H = 0xF8;	
		TMR3L = 0x31;

		++tmsecs3;
		if(++msecs3 == 1000)
		{
			msecs3 = 0;
			++secs3;
			++tsecs3;
			//err_led ^= 1;
		}
		if(secs3 == 60)
		{
			secs3 = 0;
			++mins3;
		}
		if(mins3 == 60)
		{
			mins3 = 0;
			++hours3;
		}
	}
//SERVICE THE EUSART
 if(RCIF)
 {
   usart_data = RCREG;                   // read the received usart_data 
                                // calculate buffer index 
   tmphead = ( EUSART_RxHead + 1 ) & EUSART_RX_BUFFER_MASK;
   EUSART_RxHead = tmphead;        // store new index 

   if ( tmphead == EUSART_RxTail )
   {
     // ERROR! Receive buffer overflow 
   }
                                
   EUSART_RxBuf[tmphead] = usart_data;   // store received usart_data in buffer 
  }

  
  if(TRMT)
  {
                                // check if all usart_data is transmitted  
   if ( EUSART_TxHead != EUSART_TxTail )
   {
                                // calculate buffer index  
     tmptail = ( EUSART_TxTail + 1 ) & EUSART_TX_BUFFER_MASK;
     EUSART_TxTail = tmptail;      // store new index  

    TXREG = EUSART_TxBuf[tmptail];  // start transmitiON  
   }
   else
   {
     TXIE = 0;         // disable TX interrupt  
   }
  }
}

//******************************************************************
//*	GET FIRMWARE VERSION
//*
//* ENTRY - NONE
//*
//* RETURN - results in reply structure
//******************************************************************
void get_firmware_version(void)
{
	char embedrf_reply;
		
	sendchar(0x56);
	while(!(CharInQueue()));
	if(CharInQueue())
		embedrf_reply = recvchar();
	if(embedrf_reply == 0x76)
	{
		while(!(CharInQueue()));
		if(CharInQueue())
			reply.ver_maj = recvchar();
		while(!(CharInQueue()));
		if(CharInQueue())
			reply.ver_min = recvchar();
	}
	
	delay_ms(10);
	
}
//******************************************************************
//*	GET TRANSMIT INTERVAL
//*
//* ENTRY - NONE
//*
//* RETURN - TRANSMIT INTERVAL 0x05 TO 0xFF
//* EACH BIT = 0.05 SECOND INCREMENT
//******************************************************************
void get_tx_interval(void)
{
	char embedrf_reply;
	
	sendchar(0x48);

	while(!(CharInQueue()));
	if(CharInQueue())
		embedrf_reply = recvchar();
	if(embedrf_reply == 0x68)
	{
		while(!(CharInQueue()));
		if(CharInQueue())
			reply.tx_interval = recvchar();
	}
	
	delay_ms(10);
}		
//******************************************************************
//*	SET TRANSMIT INTERVAL
//*
//* ENTRY - BYTE TRANSMIT INTERVAL
//* TRANSMIT INTERVAL 0x05 TO 0xFF
//* EACH BIT = 0.05 SECOND INCREMENT
//*
//* RETURN - 0 IF OK - 1 IF ERROR
//******************************************************************
char set_tx_interval(char interval)
{
	char embedrf_reply,frc;
	
	frc = 1;
	
	if(interval < 5)
		interval = 5;

	sendchar(0x49);
	sendchar(interval);
	
	while(!(CharInQueue()));
	if(CharInQueue())
		embedrf_reply = recvchar();
	if(embedrf_reply == 0x69)
	{
		while(!(CharInQueue()));
		if(CharInQueue())
			embedrf_reply = recvchar();
		if(embedrf_reply == interval)
			frc = 0;
	}
	delay_ms(10);
		
	return frc;
}		
//******************************************************************
//*	SET TRANSMIT/RECEIVE MODE
//*
//* ENTRY - BYTE
//* Transmit/Receive Mode Configuration Options
//* tr_off			0x00	//off - no transmit or receive
//* tr_rcv			0x01	//receive only
//* tr_xmit			0x02	//transmit only
//* tr_rx			0x03	//receive then transmit
//* tr_xr			0x04	//transmit then receive
//*
//* RETURN - 0 IF OK - 1 IF ERROR
//******************************************************************
char set_tx_rx_mode(char mode)
{
	char frc,embedrf_reply;
	
	frc = 1;
	sendchar(0x54);
	sendchar(mode);

	while(!(CharInQueue()));
	if(CharInQueue())
		embedrf_reply = recvchar();
	if(embedrf_reply == 0x74)
	{
		while(!(CharInQueue()));
		if(CharInQueue())
			embedrf_reply = recvchar();
		if(embedrf_reply == mode)
			frc = 0;
	}
	delay_ms(10);
		
	return frc;
}		
//******************************************************************
//*	GET TRANSMIT/RECEIVE MODE
//*
//* ENTRY - NONE
//*
//* RETURN - BYTE
//* Transmit/Receive Mode Configuration Options
//* tr_off			0x00	//off - no transmit or receive
//* tr_rcv			0x01	//receive only
//* tr_xmit			0x02	//transmit only
//* tr_rx			0x03	//receive then transmit
//* tr_xr			0x04	//transmit then receive
//******************************************************************
void get_tx_mode(void)
{
	char embedrf_reply;
	
	sendchar(0x58);
	
	while(!(CharInQueue()));
	if(CharInQueue())
		embedrf_reply = recvchar();
	if(embedrf_reply == 0x78)
	{
		while(!(CharInQueue()));
		if(CharInQueue())
			reply.tx_mode = recvchar();
	}
	delay_ms(10);
		
}
//******************************************************************
//*	GET RECEIVED DATA FROM EmbedRF
//*
//* ENTRY - NONE
//*
//* RETURN - 15 BYTES - RECEIVED DATA IN packet_rx
//******************************************************************
void get_embedrf_data(void)
{
	char i,embedrf_reply;
	
	sendchar(0x47);
	
	while(!(CharInQueue()));
	if(CharInQueue())
		embedrf_reply = recvchar();
	if(embedrf_reply == 0x67)
	{
		for(i=0;i<rx_end;++i)
		{
			while(!(CharInQueue()));
			if(CharInQueue())
				packet_rx[i] = recvchar();
		}
	}
	delay_ms(10);
}
//******************************************************************
//*	SEND DATA IMMEDIATELY
//*
//* ENTRY - BYTE
//* Send Data Immediately Options
//*	xmit_only	0x00
//*	xmit_rcv		0x01
//*
//* RETURN - 0 IF OK - 1 IF ERROR
//******************************************************************
char send_data_now(char cmd)
{
	char frc,embedrf_reply;
	
	frc = 1;
	
	sendchar(0x43);
	sendchar(cmd);
	
	while(!(CharInQueue()));
	if(CharInQueue())
		embedrf_reply = recvchar();
	if(embedrf_reply == 0x63)
			frc = 0;
	delay_ms(10);
			
	return frc;
}		
//******************************************************************
//*	SEND DATA TO EmbedRF BUFFER
//*
//* ENTRY - 11 BYTES - DATA IN packet_tx
//* Send Data to Buffer Options (final byte)
//* no_xmit				0x00
//* xmit_now			0x01
//*
//* RETURN - 0 IF OK - 1 IF ERROR
//******************************************************************
char send_data_buffer(char mode)
{
	char i,frc,embedrf_reply;
	
	frc = 1;
	
	sendchar(0x53);
	packet_tx[tx_cmd] = mode;
	
	for(i=0;i<tx_end;++i)
		sendchar(packet_tx[i]);
		
	while(!(CharInQueue()));
	if(CharInQueue())
		embedrf_reply = recvchar();
	if(embedrf_reply == 0x73)
		frc = 0;
	delay_ms(10);
			
	return frc;
}		
//******************************************************************
//*	SET TRANSMIT POWER LEVEL
//*
//* ENTRY - BYTE
//* Transmit Power Level Configuration Options
//* dbm_minus10		0x00	//-10 dbm
//* dbm_minus5		0x01	//-5 dbm
//* dbm_0			0x02	//0 dbm
//* dbm_plus5		0x03	//5 dbm
//*
//* RETURN - 0 IF OK - 1 IF ERROR
//******************************************************************
char set_tx_pwr_lvl(char level)
{
	char frc,embedrf_reply;
	
	frc = 1;
	sendchar(0x50);
	sendchar(level);
	
	while(!(CharInQueue()));
	if(CharInQueue())
		embedrf_reply = recvchar();
	if(embedrf_reply == 0x70)
	{
		while(!(CharInQueue()));
		if(CharInQueue())
			embedrf_reply = recvchar();
		if(embedrf_reply == level)
			frc = 0;
	}
	delay_ms(10);
		
	return frc;
}		
//******************************************************************
//*	GET TRANSMIT POWER LEVEL
//*
//* ENTRY - NONE
//*
//* RETURN - BYTE - POWER LEVEL
//******************************************************************
void get_tx_pwr_lvl(void)
{
	char embedrf_reply;
	
	sendchar(0x4F);
	
	while(!(CharInQueue()));
	if(CharInQueue())
		embedrf_reply = recvchar();
	if(embedrf_reply == 0x6F)
	{
		while(!(CharInQueue()));
		if(CharInQueue())
			reply.tx_pwr_lvl = recvchar();
	}
		
	delay_ms(10);
}
			
//******************************************************************
//*	SET DEVICE ID
//*
//* ENTRY - 24-BITS - MAX 16,777,215 (0x00FFFFFF)
//* MODE IS TRANSMIT OR RECEIVE
//* Device ID Specifier Definition
//* id_rcv			0x00
//* id_xmit			0x01
//*
//* RETURN - 0 IF OK - 1 IF ERROR
//******************************************************************
char set_device_id(char id_ww, char id_xx, char id_yy, char id_zz)
{
	char frc,embedrf_reply;
	
	frc = 0;
	
	sendchar(0x55);			
	sendchar(id_ww);
	sendchar(id_xx);
	sendchar(id_yy);
	sendchar(id_zz);
	
	while(!(CharInQueue()));
	if(CharInQueue())
		embedrf_reply = recvchar();
	if(embedrf_reply == 0x75)
	{
		while(!(CharInQueue()));
		if(CharInQueue())
		{
			if(recvchar() != id_ww)
				frc = 1;
		}
		while(!(CharInQueue()));
		if(CharInQueue())
		{
			if(recvchar() != id_xx)
				frc = 1;
		}
		while(!(CharInQueue()));
		if(CharInQueue())
		{
			if(recvchar() != id_yy)
				frc = 1;
		}
	}
	delay_ms(10);
		
	return frc;
}		
//******************************************************************
//*	GET DEVICE ID
//*
//* ENTRY 
//*Device ID Specifier Definition
//* id_rcv			0x00
//* id_xmit			0x01
//*
//* RETURN - results in reply structure
//******************************************************************
void get_device_id(char device)
{
	char embedrf_reply;
	
	
	sendchar(0x57);	
	sendchar(device);		
	
	while(!(CharInQueue()));
	if(CharInQueue())
		embedrf_reply = recvchar();
	if(embedrf_reply == 0x77)
	{
		while(!(CharInQueue()));
		if(CharInQueue())
			reply.device_id_aa = recvchar();
		while(!(CharInQueue()));
		if(CharInQueue())
			reply.device_id_bb = recvchar();
		while(!(CharInQueue()));
		if(CharInQueue())
			reply.device_id_cc = recvchar();
	}
	delay_ms(10);
}		
//******************************************************************
//*	SET ANALOG/DIGITAL SETTINGS
//*
//* ENTRY 
//* Analog/Digital Options
//*	ad_disabled		0x00	//disabled
//*	ad_enabled		0x01	//analog active
//*	dig_1			0x02	//digital active logic 1
//* dig_0			0x03	//digital active logic 0
//*
//* RETURN - 0 IF OK - 1 IF ERROR
//******************************************************************
char set_ad_settings(char ad0val, char ad1val, char ad2val, char ad3val)
{
	char frc,embedrf_reply;
	
	frc = 0;
	
	sendchar(0x41);			
	sendchar(ad0val);	
	sendchar(ad1val);	
	sendchar(ad2val);	
	sendchar(ad3val);	

	while(!(CharInQueue()));
	if(CharInQueue())
		embedrf_reply = recvchar();
	if(embedrf_reply == 0x61)
	{
		while(!(CharInQueue()));
		if(CharInQueue())
		{
			embedrf_reply = recvchar();
			if(embedrf_reply != ad0val)
				frc = 1;
		}
		while(!(CharInQueue()));
		if(CharInQueue())
		{
			embedrf_reply = recvchar();
			if(embedrf_reply != ad1val)
				frc = 1;
		}
		while(!(CharInQueue()));
		if(CharInQueue())
		{
			embedrf_reply = recvchar();
			if(embedrf_reply != ad2val)
				frc = 1;
		}
		while(!(CharInQueue()));
		if(CharInQueue())
		{
			embedrf_reply = recvchar();
			if(embedrf_reply != ad3val)
				frc = 1;
		}
	}
	delay_ms(10);
		
	return frc;
}		
//******************************************************************
//*	GET ANALOG/DIGITAL SETTINGS
//*
//* ENTRY - NONE
//*
//* RETURN - results in reply structure
//******************************************************************
void get_ad_settings(void)
{
	char embedrf_reply;
	
	sendchar(0x42);			
	
	while(!(CharInQueue()));
	if(CharInQueue())
		embedrf_reply = recvchar();
	if(embedrf_reply == 0x62)
	{
		while(!(CharInQueue()));
		if(CharInQueue())
			reply.ad_setting_ww = recvchar();
		while(!(CharInQueue()));
		if(CharInQueue())
			reply.ad_setting_xx = recvchar();
		while(!(CharInQueue()));
		if(CharInQueue())
			reply.ad_setting_yy = recvchar();
		while(!(CharInQueue()));
		if(CharInQueue())
			reply.ad_setting_zz = recvchar();
	}
	delay_ms(10);
}		
//******************************************************************
//*	FIND TRANSMITTER
//*
//* ENTRY - NONE
//* Find Transmitter Options
//*	find_first		0x00	//find first
//* prox_find		0x01	//proximity find
//* RETURN - results in reply structure
//******************************************************************
void find_xmitr(char mode)
{
	char embedrf_reply;
	
	sendchar(0x46);			
	sendchar(mode);			
	
	while(!(CharInQueue()));
	if(CharInQueue())
		embedrf_reply = recvchar();
	if(embedrf_reply == 0x66)
	{
		while(!(CharInQueue()));
		if(CharInQueue())
			reply.find_tx_xx = recvchar();
		while(!(CharInQueue()));
		if(CharInQueue())
			reply.find_tx_yy = recvchar();
		while(!(CharInQueue()));
		if(CharInQueue())
			reply.find_tx_zz = recvchar();
	}
	delay_ms(10);
}		

//******************************************************************
//*	SET NETWORK ID
//*
//* ENTRY - 24-bit Network ID (0x000000 - 0xFFFFFF)
//*
//* RETURN - 0 IF OK - 1 IF ERROR
//******************************************************************
char set_network_id(char nid_xx, char nid_yy, char nid_zz)
{
	char frc,embedrf_reply;
	
	frc = 0;
	
	sendchar(0x4D);			 
	sendchar(nid_xx); 
	sendchar(nid_yy); 
	sendchar(nid_zz); 
	
	while(!(CharInQueue()));
	if(CharInQueue())
		embedrf_reply = recvchar();
	if(embedrf_reply == 0x6D)
	{
		while(!(CharInQueue()));
		if(CharInQueue())
		{
			if(recvchar() != nid_xx)
				frc = 1;
		}
		while(!(CharInQueue()));
		if(CharInQueue())
		{
			if(recvchar() != nid_yy)
				frc = 1;
		}
		while(!(CharInQueue()));
		if(CharInQueue())
		{
			if(recvchar() != nid_zz)
				frc = 1;
		}
	}
		
	delay_ms(10);
	return frc;
}	
//******************************************************************
//*	SET NOTIFY MODE
//*
//* ENTRY - 
//* Notify Mode Options
//* notify_off		0x00
//*	notify_on		0x01
//* RETURN - 0 IF OK - 1 IF ERROR
//******************************************************************
char set_notify_mode(char mode)
{
	char frc,embedrf_reply;
	
	frc = 1;
	
	sendchar(0x4E);			
	sendchar(mode);			
	
	while(!(CharInQueue()));
	if(CharInQueue())
		embedrf_reply = recvchar();
	if(embedrf_reply == 0x6E)
	{
		while(!(CharInQueue()));
		if(CharInQueue())
			embedrf_reply = recvchar();
		
		if(embedrf_reply == mode)
			frc = 0;
	}
	
	delay_ms(10);
	return frc;

}
//******************************************************************
//*	SET DEVICE ID TO PSEUDO-RANDOM NUMBER
//*
//* ENTRY - 
//* Pseudo-Random Options
//*	rcv_random		0x00
//*	xmit_random		0x01
//* 
//* RETURN - results in reply structure
//******************************************************************
void set_id_random(char device)
{
	char embedrf_reply;
	
	
	sendchar(0x44);			
	sendchar(device);			
	
	while(!(CharInQueue()));
	if(CharInQueue())
		embedrf_reply = recvchar();
	if(embedrf_reply == 0x64)
	{
		while(!(CharInQueue()));
		if(CharInQueue())
			reply.rid_xx = recvchar();
		while(!(CharInQueue()));
		if(CharInQueue())
			reply.rid_yy = recvchar();
		while(!(CharInQueue()));
		if(CharInQueue())
			reply.rid_zz = recvchar();
	}
	delay_ms(10);
}		

//******************************************************************
//*	SET NETWORK MODE
//*
//* ENTRY - 
//* Network Mode Options
//*	p_to_p			0x00	//point to point
//* p_to_m			0x01	//point to multipoint//* RETURN - 0 IF OK - 1 IF ERROR
//* 
//* RETURN - 0 IF OK - 1 IF ERROR
//******************************************************************
char set_network_mode(char mode)
{
	char frc,embedrf_reply;
	
	frc = 1;
	
	sendchar(0x4B);			
	sendchar(mode);			
	
	while(!(CharInQueue()));
	if(CharInQueue())
		embedrf_reply = recvchar();
	if(embedrf_reply == 0x6B)
	{
		while(!(CharInQueue()));
		if(CharInQueue())
			embedrf_reply = recvchar();
		
		if(embedrf_reply == mode)
			frc = 0;
	}
	
	delay_ms(10);
	return frc;

}
//******************************************************************
//*	GET NETWORK MODE
//*
//* ENTRY - NONE
//* RETURN - results in reply structure
//******************************************************************
void get_network_mode(void)
{
	char embedrf_reply;
	
	sendchar(0x4A);
	
	while(!(CharInQueue()));
	if(CharInQueue())
		embedrf_reply = recvchar();
	if(embedrf_reply == 0x6A)
	{
		while(!(CharInQueue()));
		if(CharInQueue())
			reply.network_mode = recvchar();
	}
		
	delay_ms(10);
}
//****************************************************************************
//*	INITIALIZE ROUTINE
//****************************************************************************
void init()
{
//****************************************************************************
//*	INITIALIZE CLOCK AND IO PORTS
//****************************************************************************
	OSCCON = 0x70;		//8 MHz internal clock
	PLLEN = 0;			//PLL OFF
	TRISA = 0b11011111;
	TRISB = 0b11111111;
	TRISC = 0b11111111;
//******************************************************************
//*	CONFIGURE A2D AND COMPARATORS
//******************************************************************
	ADCON1 = 0b00001111;	//all digital inputs
	ADON = 0;				//A2D OFF
	CMCON = 0x07;			//comparators OFF
//****************************************************************************
//*	INITIALIZE USART
//****************************************************************************
	init_EUSART();
//******************************************************************
//*	CONFIGURE AND START TIMER3
//* SET TO OVERFLOW EVERY 1mS
//******************************************************************
	//F831 = 8MHz
	//E0C1 = 32MHz
 	hours3 = 12;
	mins3 = 0;
	secs3 = 0;
  	msecs3 = 0;
	T3CON = 0x00;		
	TMR3H = 0xF8;	
	TMR3L = 0x31;
	TMR3ON = 1;
//******************************************************************
//*	CONFIGURE INTERRUPTS
//******************************************************************
	enable_TMR3int;
	enable_GLOBALint;
}
		
//******************************************************************
//*	MAIN SERVICE LOOP
//* YOUR APPLICATION CODE GOES HERE..
//******************************************************************
void main(void)
{
	char i;
	
	init();
	cleaner();
	for(i=0;i<tx_end;++i)
		packet_tx[i] = i+2;
		
	err_led = OFF;
	//if(rc = set_notify_mode(notify_on))
	//	err_led = ON;
	if(rc = set_tx_interval(0x28))
		err_led = ON;
	if(rc = set_tx_rx_mode(tr_xmit))
		err_led = ON;
	if(rc = set_device_id(0x11, 0x22, 0x33, id_xmit))
		err_led = ON;
	//set_id_random(tx_device);
	if(rc = set_network_id(0x00,0x00,0x00))
		err_led = ON;
	if(rc = set_ad_settings(0x00,0x00,0x00,0x00))
		err_led = ON;
	if(rc = set_network_mode(p_to_p))
		err_led = ON;
	if(rc = set_tx_pwr_lvl(dbm_plus5))
		err_led = ON;
	if(rc = send_data_buffer(no_xmit))
		err_led = ON;
	//cleaner();
	//find_xmitr(0x00);
/*	
	get_firmware_version();
	get_tx_interval();
	get_tx_mode();
	get_device_id(rx_device);
	get_ad_settings();
	get_network_mode();
	get_tx_pwr_lvl();


		while(!(CharInQueue()));
		for(i=0;i<rx_end;++i)
		{
			while(!(CharInQueue()));
			if(CharInQueue())
				packet_rx[i] = recvchar();
		}
*/
		do{
		++i;
		}while(1);
}